#include "gbox.h"
#include "gbox_mailbox.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

extern MailBoxList *mb_list;
GboxApplet  *applet;
MailBox     *exchange_mb;
GtkWidget   *tip_list, *scroll_win;
gint        tooltip_height, tooltip_width;

void show_tooltip(GtkButton *button, gpointer user_data) {
  GtkRequisition req;
  GtkWidget      *box;
  gint x, y;

  update_tooltip();
  
  gtk_clist_set_shadow_type(GTK_CLIST(tip_list), GTK_SHADOW_ETCHED_OUT);
  gtk_clist_set_column_auto_resize (GTK_CLIST(tip_list), 0, TRUE);
  
  gtk_clist_set_selection_mode(GTK_CLIST(tip_list), GTK_SELECTION_SINGLE);
  gtk_clist_set_column_width(GTK_CLIST(tip_list), 0, gtk_clist_optimal_column_width(GTK_CLIST(tip_list), 0));
  gtk_clist_set_column_width(GTK_CLIST(tip_list), 1, gtk_clist_optimal_column_width(GTK_CLIST(tip_list), 1));
  
  if (!applet->show_tooltip) return;

  gtk_widget_get_child_requisition(GTK_WIDGET(applet->gbox_applet), &req);
  gdk_window_get_origin(applet->gbox_applet->window, &x, &y);
  
  applet->tip = gtk_window_new(GTK_WINDOW_POPUP);
  gtk_container_set_border_width(GTK_CONTAINER(applet->tip), 0);

  box = gtk_hbox_new(0, 0);
  
  gtk_widget_show(GTK_WIDGET(tip_list));
  
  gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(tip_list), TRUE, TRUE, 0);
  gtk_widget_show(GTK_WIDGET(box));
  
  gtk_container_add(GTK_CONTAINER(applet->tip), GTK_WIDGET(box));
  
  /*  gtk_widget_get_child_requisition(GTK_WIDGET(tip_list), &req);*/
  
  if ((x + tooltip_width) > gdk_screen_width())
    gtk_widget_popup(GTK_WIDGET(applet->tip), gdk_screen_width() - 2*12 - tooltip_width, y - tooltip_height);
  else
    gtk_widget_popup(GTK_WIDGET(applet->tip), x, y - tooltip_height);

  /*  gtk_widget_get_child_requisition(GTK_WIDGET(tip_list), &req);*/
}

void hide_tooltip(GtkButton *button, gpointer user_data) {
  if (!applet->show_tooltip) return;
  gtk_widget_destroy(GTK_WIDGET(applet->tip));
  tip_list = NULL;
}

void row_length(char **text) {
  GtkStyle *style = gtk_widget_get_default_style();

  
  if (tooltip_width < (gdk_string_width(style->font, text[0]) + 
		       gdk_string_width(style->font, text[1]))) {
      tooltip_width = gdk_string_width(style->font, text[0])
	+ gdk_string_width(style->font, text[1]);
  }
  
}
  
void update_tooltip() {
  gchar    *text[2];
  GtkStyle *style = gtk_widget_get_default_style();
  int      gap, baseline_skip, line_height;
  int      row_count = 0;
  MailBox  *walk = mb_list->first_mb;
  GdkColor hilight_color = { 0, 0xCCDD, 0x1111, 0x1111 };
  
  tooltip_height = tooltip_width = 0;
  gap = (style->font->ascent + style->font->descent) / 4;
  if (gap < 2)
    gap = 2;
  
  line_height = gdk_string_height(style->font, "a") + style->font->ascent + style->font->descent;
  
  baseline_skip = style->font->ascent + style->font->descent + gap;
  
  if (!tip_list) tip_list = gtk_clist_new(2);
  gtk_clist_clear(GTK_CLIST(tip_list));
  
  switch (applet->style_tooltip) {
  case 0:
    text[0] = _("watched mailboxes");
    tooltip_height += line_height;
    text[1] = g_strdup_printf("%d", count_mailboxes());
    gtk_clist_append(GTK_CLIST(tip_list), text);
    row_length(text);
    
    /*    if (tooltip_width < (gdk_string_width(style->font, text[0]) + gdk_string_width(style->font, text[1])))
	  tooltip_width = gdk_string_width(style->font, text[0]) + gdk_string_width(style->font, text[1]);*/
    
    g_free(text[1]);
    
    text[0] = _("contain new mail");
    tooltip_height += line_height;
    text[1] = g_strdup_printf("%d", count_mailboxes_with_new_mail());
    gtk_clist_append(GTK_CLIST(tip_list), text);

    row_length(text);

    g_free(text[1]);
    break;
  case 1:
    text[0] = _("highest priority");
    tooltip_height += line_height;
    text[1] = g_strdup_printf("%d", mb_list->new_count[0]);
    gtk_clist_append(GTK_CLIST(tip_list), text);
    row_length(text);
    
    g_free(text[1]);
    
    text[0] = _("middle priority");
    tooltip_height += line_height;
    text[1] = g_strdup_printf("%d", mb_list->new_count[1]);
    gtk_clist_append(GTK_CLIST(tip_list), text);
    row_length(text);
    
    g_free(text[1]);

    text[0] = _("lowest priority");
    tooltip_height += line_height;
    text[1] = g_strdup_printf("%d", mb_list->new_count[2]);
    gtk_clist_append(GTK_CLIST(tip_list), text);
    row_length(text);
    
    g_free(text[1]);
    break;
  case 2:
    while (walk) {
      text[0] = walk->nickname;
      tooltip_height += line_height;
      text[1] = g_strdup_printf("%d", walk->mail_count);
      
      gtk_clist_append(GTK_CLIST(tip_list), text);
      row_length(text);
      
      if (walk->mail_in_box)
	gtk_clist_set_foreground(GTK_CLIST(tip_list), row_count, &hilight_color);
      
      g_free(text[1]);
      walk = walk->next;row_count++;
    }
    break;
  case 3:
    while (walk) {
      if (walk->mail_in_box == TRUE) {
	text[0] = walk->nickname;
	tooltip_height += line_height;
	text[1] = g_strdup_printf("%d", walk->mail_count);
	
	gtk_clist_append(GTK_CLIST(tip_list), text);
	row_length(text);
	
	g_free(text[1]);
	row_count++;
      }
      walk = walk->next;
    }
  }

  tooltip_height += 12 + row_count;
  
  gtk_clist_set_row_height(GTK_CLIST(tip_list), line_height);
}

/*
  -3 - error
  -2 - size of mailbox became zero
  -1 - size of mailbox is less than before
   0 - size of mailbox is zero
   1 - size of mailbox is more than before
   2 - size of mailbox does not change but is not zero
   3 - new mail arrived in an empty mailbox
 */
int check_mailbox(MailBox *mb) {
    MailBox *walk = mb;
    struct stat buff;
    int    mail = 0;
    
    if (!lstat(walk->mailfile, &buff)) {
	if (buff.st_size == walk->size)
	    mail = (!walk->size?0:2);
	else {
	    if (walk->size < buff.st_size) {
		if (!walk->size) mail = 3;
		else mail = 1;
	    } else {
		if (walk->size > buff.st_size) {
		    if (!buff.st_size) mail = -2;
		    else mail = -1;
		}
	    }
	}
	
	walk->size = buff.st_size;
    } else {
	mail = -3;
    }
	
    walk->mail_count = count_mails_in_box(walk->mailfile);

    switch (mail) {
    case -2:
	(mb_list->new_count[walk->priority])--;
	walk->mail_in_box = FALSE;
	break;
    case 3:
	(mb_list->new_count[walk->priority])++;
	walk->mail_in_box = TRUE;
	if (applet->style_beep)
	  gdk_beep();
    }

    return mail;
}

void update_mailboxes_cb(AppletWidget *app, gpointer data) {
  MailBox * walk = mb_list->first_mb;
  
  while (walk) {
    check_mailbox(walk);
    walk = walk->next;
  }

  set_mail_icon();
}

void set_mail_icon() {
  int found_new = FALSE;
  int i;
  
  for (i = 0; i < COUNT_PRIORITY; i++)
    if (mb_list->new_count[i] > 0) {
      gtk_pixmap_set(GTK_PIXMAP(applet->image_win), 
		     applet->new_email_pixmap[i], 
		     applet->new_email_mask[i]);
      found_new = TRUE;break;
    }

  /*  update_tooltip();*/
  
  if (!found_new)
    gtk_pixmap_set(GTK_PIXMAP(applet->image_win), applet->no_email_pixmap, applet->no_email_mask);
}

gint timeout_callback( gpointer data ) {
  /*  printf("CALLBACK: %s | %d\n", 
	 ((MailBox *)data)->mailfile, 
	 mb_list->new_count[((MailBox *)data)->priority]);*/
  
  check_mailbox((MailBox *) data);
  
  set_mail_icon();
  
  return 1; /* 0 will stop the timer forever */
}

static gint exec_mail_prog(GtkWidget *widget, GdkEvent *evt, gpointer data) {
    if (applet->mail_prog && (strlen(applet->mail_prog)) > 0)
	gnome_execute_shell(NULL, applet->mail_prog);

    return 1;
}

void start_timer() {
    MailBox *walk = mb_list->first_mb;
    
    while (walk) {
      	check_mailbox(walk);

	/*	printf("interval: %d (%s)\n", walk->interval, walk->mailfile);*/
	
	walk->timer = gtk_timeout_add( walk->interval,
				       timeout_callback,
				       walk);
	walk = walk->next;
    }
}

void stop_timer() {
    MailBox *walk = mb_list->first_mb;
    
    while (walk) {
	gtk_timeout_remove(walk->timer);
	walk = walk->next;
    }
}

static void mailcheck_destroy (GtkWidget *widget, gpointer data) {
    gtk_main_quit();
}

void load_image(gchar *file, GdkImlibImage **image, GdkPixmap **pixmap, GdkBitmap **mask) {
  GdkImlibImage *im;
  GdkPixmap     *mk, *px;

  im = *image;
  px = *pixmap;
  mk = *mask;
  
  *image = gdk_imlib_load_image(file);

  gdk_imlib_render(*image, 
		   (*image)->rgb_width, 
		   (*image)->rgb_height); 
  *pixmap = gdk_imlib_move_image(*image);
  *mask = gdk_imlib_move_mask(*image);

  g_free(im);
  /*  g_free(px);
      g_free(mk); */
}

static GtkWidget *create_mail_widgets (GtkWidget *parent) {
  GtkWidget *box;
  int       i;
  
  applet->no_email_image = NULL;
  applet->no_email_pixmap = NULL;
  applet->no_email_mask = NULL;
  
  box = gtk_hbox_new (0, 0);
  applet->main_button = gtk_button_new();
  gtk_signal_connect (GTK_OBJECT (box), "destroy",
		      (GtkSignalFunc) mailcheck_destroy,
		      NULL);
  
  gtk_widget_show (box);
  
  load_image(applet->no_mail_pix, 
	     &applet->no_email_image, &applet->no_email_pixmap, &applet->no_email_mask);
  
  for (i = 0; i < COUNT_PRIORITY; i++) {
    if ((applet->new_email_image[i] = gdk_imlib_load_image(applet->new_mail_pix[i])) == NULL) {
      printf("Ooops: please, do not give any non-existing image-file!!\n!");
      exit(1);
    }
  
    gdk_imlib_render(applet->new_email_image[i], 
		     applet->new_email_image[i]->rgb_width, 
		     applet->new_email_image[i]->rgb_height);
    applet->new_email_pixmap[i] = gdk_imlib_move_image(applet->new_email_image[i]);
    applet->new_email_mask[i] = gdk_imlib_move_mask(applet->new_email_image[i]);
  }
  
  applet->image_win = gtk_pixmap_new( applet->no_email_pixmap, applet->no_email_mask );
  gtk_container_add(GTK_CONTAINER(applet->main_button), applet->image_win);
  
  if (applet->style_relief)
    gtk_button_set_relief(GTK_BUTTON(applet->main_button), GTK_RELIEF_NONE);
  
  gtk_widget_show(applet->main_button);
  gtk_signal_connect( GTK_OBJECT(applet->main_button), "clicked",
		      GTK_SIGNAL_FUNC(exec_mail_prog), 
		      NULL);
  
  gtk_box_pack_start(GTK_BOX(box), applet->main_button, FALSE, FALSE, 0);
  gtk_widget_show(applet->image_win);
  gtk_widget_show(box);
  
  return box;
}
static void about_cb(AppletWidget* w, gpointer cbdata)
{
  GtkWidget* about_box;
  
  static const gchar* author[] = { "Andreas Bsching (crunchy@tzi.de) - programming", 
				   "Thomas Buntrock (bunti@tzi.de) - graphics & testing", 
				   "Dirk Meyer (dmeyer@tzi.de) - testing", 
				   NULL };
  
 about_box = gnome_about_new(_(PACKAGE), VERSION,
			      "(C) 1999 Andreas Bsching",
			      author,
			      _("Watching multiply mbox files ..."),
			      NULL);
  
  gtk_widget_show(about_box);
}

static void save_session_info(void)
{
  MailBox *walk = mb_list->first_mb;
  gchar box[128];
  int i;
  
  gnome_config_push_prefix(APPLET_WIDGET(applet->gbox_applet)->privcfgpath);
  
  gnome_config_set_string("/gbox_applet/global/no_mail_pixmap", applet->no_mail_pix);
  gnome_config_set_string("/gbox_applet/global/mail_prog", applet->mail_prog);
  gnome_config_set_bool("/gbox_applet/global/beep", applet->style_beep);
  gnome_config_set_bool("/gbox_applet/global/relief", applet->style_relief);
  gnome_config_set_bool("/gbox_applet/global/tooltip", applet->show_tooltip);
  gnome_config_set_int("/gbox_applet/global/tooltip_style", applet->style_tooltip);
  gnome_config_set_int("/gbox_applet/global/count_mailbox", count_mailboxes());
  for (i = 0;i < count_mailboxes(); i++) {
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/filename", i);
    gnome_config_set_string(box, walk->mailfile);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/nickname", i);
    gnome_config_set_string(box, walk->nickname);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/interval", i);
    gnome_config_set_int(box, walk->interval);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/priority", i);
    gnome_config_set_int(box, walk->priority);
    walk = walk->next;
  }
  
  gnome_config_set_int("/gbox_applet/priority/count", COUNT_PRIORITY);
  for (i = 0;i < COUNT_PRIORITY; i++) {
    snprintf(box, sizeof(box), "/gbox_applet/priority/prio-%d", i);
    gnome_config_set_string(box, applet->new_mail_pix[i]);
    walk = walk->next;
  }
  
  gnome_config_pop_prefix();
  gnome_config_sync();
  gnome_config_drop_all();
}

static void quit_applet(GtkWidget* w, gpointer cbdata)
{
  save_session_info();
  applet_widget_gtk_main_quit();
}

static gint applet_save_session(GtkWidget* w, const gchar* priv,
						 const gchar* global)
{
   save_session_info();
   return FALSE;
}

static void read_config_data(gchar* cfgpath)
{
   gchar* tmp, *nick, *help;
   int    count_mb = 0, i, num, prio, count_prio = 0;
   char   box[128];
   

   if(cfgpath == NULL)
	 return;
   gnome_config_push_prefix(cfgpath);
   
   tmp = gnome_config_get_string("/gbox_applet/global/no_mail_pixmap=" GBOX_PIXMAP_DIR "/default-no-mail.xpm");
   applet->no_mail_pix = g_strdup(tmp);
   g_free(tmp);

   tmp = gnome_config_get_string("/gbox_applet/global/mail_prog=emacs -f gnus");
   applet->mail_prog = g_strdup(tmp);
   g_free(tmp);

   applet->style_beep = gnome_config_get_bool("/gbox_applet/global/beep=false");
   applet->style_relief = gnome_config_get_bool("/gbox_applet/global/relief=false");
   applet->show_tooltip = gnome_config_get_bool("/gbox_applet/global/tooltip=false");   
   applet->style_tooltip = gnome_config_get_int("/gbox_applet/global/tooltip_style=0");
   count_mb = gnome_config_get_int("/gbox_applet/global/count_mailbox=0");
   for (i = 0; i < count_mb; i++) {
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/filename", i);
     tmp = gnome_config_get_string(box);
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/nickname", i);
     nick = gnome_config_get_string(box);
     if (!nick) {
       help = strrchr(tmp, '/');
       if (help) nick = help + 1;
     }
     
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/interval", i);
     num = gnome_config_get_int(box);
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/priority=%d", i, COUNT_PRIORITY - 1);
     prio = gnome_config_get_int(box);
     
     add_mailbox(tmp, nick, num, prio);
   }

   snprintf(box, (size_t) sizeof(box), "/gbox_applet/priority/count=%d", COUNT_PRIORITY);
   count_prio = gnome_config_get_int(box);
   for (i = 0; i < count_prio; i++) {
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/priority/prio-%d=" 
	      GBOX_PIXMAP_DIR "/default-prio-%d.xpm", i, i);
     applet->new_mail_pix[i] = gnome_config_get_string(box);
   }
   
   gnome_config_pop_prefix();
}

int main(int argc, char *argv[]) {
  int i;

  applet = (GboxApplet *)malloc(sizeof(GboxApplet));
  mb_list = (MailBoxList *)malloc(sizeof(MailBoxList));
  exchange_mb = (MailBox *)malloc(sizeof(MailBox));
  exchange_mb->mailfile = NULL;
  exchange_mb->nickname = NULL;
  
  mb_list->first_mb = NULL;
  mb_list->last_mb = NULL;
  tip_list = NULL;

  
  for (i = 0; i < COUNT_PRIORITY; i++)
      mb_list->new_count[i] = 0;
  
  bindtextdomain(PACKAGE, GNOMELOCALEDIR);
  textdomain(PACKAGE);
  
  applet_widget_init(PACKAGE, VERSION, argc, argv,
		     NULL, 0, NULL); 
  // for using imlib
  gdk_imlib_init();  
  gtk_widget_push_visual(gdk_imlib_get_visual());
  gtk_widget_push_colormap(gdk_imlib_get_colormap());
  // end for using imlib

  applet->gbox_applet = applet_widget_new(PACKAGE);
  
  if (!applet->gbox_applet) {
    printf("failed to create applet!\n");
    exit(0);
  }
  
  gtk_widget_realize(applet->gbox_applet);
  
  /* reading configuration */
  read_config_data(APPLET_WIDGET(applet->gbox_applet)->privcfgpath);
  
  /* printf("PIXMAP: %s\n", applet->no_mail_pix); */
  
  applet->mailcheck = create_mail_widgets(applet->gbox_applet);
  gtk_widget_show(applet->mailcheck);
  applet_widget_add (APPLET_WIDGET (applet->gbox_applet), applet->mailcheck);
  
  gtk_widget_show(applet->gbox_applet);
  
  /* tooltip */
  /*  applet->tooltip = gtk_tooltips_new();*/
  update_tooltip();  

  gtk_signal_connect(GTK_OBJECT(applet->gbox_applet), "delete_event",
		     GTK_SIGNAL_FUNC(quit_applet), NULL);
  gtk_signal_connect(GTK_OBJECT(applet->gbox_applet), "save_session",
		     GTK_SIGNAL_FUNC(applet_save_session), NULL);
  gtk_signal_connect(GTK_OBJECT(applet->main_button), "enter",
		     GTK_SIGNAL_FUNC(show_tooltip), NULL);
  gtk_signal_connect(GTK_OBJECT(applet->main_button), "leave",
		     GTK_SIGNAL_FUNC(hide_tooltip), NULL);

  
  applet_widget_register_stock_callback (APPLET_WIDGET(applet->gbox_applet),
					 "properties",
					 GNOME_STOCK_MENU_PROP,
					 _("properties"),
					 applet_properties,
					 NULL);

  
  /*  applet_widget_register_callback (APPLET_WIDGET(applet->gbox_applet),
      "update",
      _("update"),
      &update_mailboxes_cb,
      NULL);*/
  
  applet_widget_register_stock_callback(APPLET_WIDGET(applet->gbox_applet),
					"update",
					GNOME_STOCK_MENU_REFRESH,
					_("update"),
					update_mailboxes_cb, NULL);

  applet_widget_register_stock_callback(APPLET_WIDGET(applet->gbox_applet),
					"about",
					GNOME_STOCK_MENU_ABOUT,
					_("About..."),
					about_cb, NULL);

  start_timer();
  set_mail_icon();
  
  applet_widget_gtk_main(); 
  
  free(mb_list);
  free(exchange_mb);
  free(applet);
  
  return 0;
}



