#include "gbox.h"
#include "gbox_mailbox.h"

#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>

MailBoxList *mb_list;

void add_mailbox(char *file, char *nick, int interval, int prio) {
    MailBox *new_mb; 

    new_mb = malloc(sizeof(MailBox));
    new_mb->next = NULL;
    new_mb->mailfile = g_strdup(file);
    new_mb->nickname = g_strdup(nick);
    new_mb->size = 0;
    new_mb->interval = interval;
    new_mb->priority = prio;
    new_mb->mail_in_box = FALSE;
    new_mb->mail_count = 0;

    if (mb_list->last_mb) mb_list->last_mb->next = new_mb;
    mb_list->last_mb = new_mb;
    if (!mb_list->first_mb) mb_list->first_mb = new_mb;
}

int count_mails_in_box(char *file) {
  FILE *f = 0;
  char buf[1024];
  char *boundary = NULL;
  char *boundary_end = NULL;
  char *pos1, *pos2;
  int count = 0;
  int ignore = 0;
  
  printf("counting in box: %s\n", file);
  
  if ((f = fopen(file,"r"))) {
    while (fgets(buf,sizeof(buf),f) != 0) {
      if (!ignore && strncmp(buf,"Subject: ",9) == 0) 
	count++;

      if (!ignore && strncmp(buf,"Content-Type: ",14) == 0 && strstr(buf,"boundary") != 0) {
	ignore = 1;
	pos1 = strchr(buf,'"');
	pos2 = strrchr(buf,'"');
	if (pos1 != 0 && pos2 > pos1+1) {
	  boundary = (char*)malloc((size_t)(pos2-pos1));
	  strncpy(boundary,pos1+1,pos2-pos1-1);
	  boundary_end = (char*)malloc((size_t)(pos2-pos1+4));
	  sprintf(boundary_end, "--%s--", boundary);
	} else ignore = 0;
	
      }
      if (ignore && strncmp(buf,boundary_end,strlen(boundary_end)) == 0) {
	ignore = 0;
	free(boundary_end);
	free(boundary);
      }
    }
    fclose (f);
  }
  printf("finished counting\n");

  return count;
}

int count_mailboxes_with_new_mail() {
  MailBox *walk = mb_list->first_mb;
  int count = 0;
  
  while (walk) {
    if (walk->mail_in_box) count++;
    walk = walk->next;
  }
  
  return count;
}
 
int count_mailboxes() {
  MailBox *walk = mb_list->first_mb;
  int count = 0;
  
  while (walk) {
    count++;
    walk = walk->next;
  }
  
  return count;
}

void change_mailbox(char *file, MailBox* new_box) {
  MailBox *walk = mb_list->first_mb;
  
  while (walk) {
    if (!strcmp(walk->mailfile, file)) break;
    walk = walk->next;
  }
  
  if (walk) {
    g_free(walk->mailfile);
    walk->mailfile = g_strdup(new_box->mailfile);
    walk->nickname = g_strdup(new_box->nickname);
    walk->interval = new_box->interval;
    walk->priority = new_box->priority;
  }
}

void remove_mailbox(char *file) {
  MailBox *walk = mb_list->first_mb, *prev = NULL;
  
  while (walk) {
    if (!strcmp(walk->mailfile, file)) break;
    
    prev = walk;
    walk = walk->next;
  }
  
  if (!walk) return;
  
  if (!prev) {   // remove first;
    mb_list->first_mb = mb_list->first_mb->next;
  } else {
    prev->next = walk->next;
  }
  
  if (mb_list->last_mb == walk) mb_list->last_mb = prev;
  
  free(walk);
}

void remove_mailbox_by_index(int index) {
  remove_mailbox(get_mailbox(index)->mailfile);
}

MailBox *get_mailbox(int index) {
  MailBox *walk = mb_list->first_mb;
  int i = 0;
  
  while (walk) {
    if (i == index) break;
    i++;
    walk = walk->next;
  }

  return walk;
}
